pro destripe, drifts_all, flag_valid_scans, map_back=map_back, smallfield=smallfield, $
              size_deg=size_deg, file_scalars=file_scalars, ind_sc_coarse=ind_sc_coarse, $
              ind_scans=ind_scans, ind_subscans=ind_subscans, maxnoise=maxnoise, $
              cut_start=cut_start, galactic=galactic, disjoint=disjoint, inner_edges=inner_edges, $
              list_scanrot=list_scanrot, time_1=time_1, t_proc=t_proc


;;; computes residual linear baselines from the redundancy

;;;;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

;;; copyright 2007-2016 H\'el\`ene Roussel
;;; roussel@iap.fr
;;; Institut d'Astrophysique de Paris - Universit\'e Pierre et Marie Curie, France

;;; download and documentation: http://www2.iap.fr/users/roussel/herschel/

;;; Scanamorphos is distributed under the terms of the GNU General Public License
;;; (http://www.gnu.org/licenses/gpl.html).

;;; July 2010: public release version 1

;;;;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~



restore, file_scalars

crs = string(13b) + string(10b)


if ((size_deg lt 1. and nscans le 20) or nt lt 50000L) then ns = 49 else ns = 0
size_lim_ref = 0.75
size_lim_smallfield = 2.

dm = round(sqrt(dimx_iter * dimy_iter))
if dm gt 500L then begin
   fact_old = fact
   fact = ceil(sqrt(dimx * dimy) / 500L)
   dimx_iter = dimx / fact
   if dimx mod fact gt 0 then dimx_iter += 1
   dimy_iter = dimy / fact
   if dimy mod fact gt 0 then dimy_iter += 1
   if keyword_set(visu) then print, 'enlarging the pixel size for destriping by a factor ' $
      + chain(fact/float(fact_old), 2)
endif


cube_signal = fltarr(dimx_iter, dimy_iter, nb, nscans)
cube_weight = fltarr(dimx_iter, dimy_iter, nb, nscans)
cube_subscan = lonarr(dimx_iter, dimy_iter, nb, nscans)
dimxy = dimx_iter * dimy_iter
dimxynb = dimxy * nb
tab_nb = dimxy * lindgen(nb)

tab_scan = lonarr(ind_scans(1))
for i = 1, nscans - 1 do tab_scan = [temporary(tab_scan), i + lonarr(ind_scans(i+1) - ind_scans(i))]

tab_subscan = lonarr(ind_subscans(1))
for i = 1L, nsubscans - 1L do tab_subscan = [temporary(tab_subscan), i + lonarr(ind_subscans(i+1) - ind_subscans(i))]
tab_subscan += 1L

restore, file_times
w = where(flag_nza lt 1, cw)
if cw ge 1 then tab_subscan(w) = 0L
for i = 0L, nsubscans - 1L do begin
   w = where(flag_nza(ind_subscans(i):ind_subscans(i+1)-1) lt 1, cw)
   if cw ge 1 then tab_subscan(ind_subscans(i)+max([0L, w(0)-200L]):min([ind_subscans(i+1)-1, ind_subscans(i)+w(cw-1)+200L])) = i + 1L
endfor

weight = 1. / maxnoise^2.


;;; remove duplicate offset grid:
prepare_proj_coarse, file_scalars=file_scalars, /onegrid, new_dimx_iter=dimx_iter, new_dimy_iter=dimy_iter, $
   new_fact=fact



restore, file_signal

restore, file_projcoarse

restore, file_vscan

restore, file_time_coarse


for k = 0L, nproj - 1L do begin
   revind_0 = revind(wproj(k))
   revind_1 = revind(wproj(k)+1)
   cp = revind_1 - revind_0
   wp = revind(revind_0:revind_1-1)
   wb = wp / nt
   ws = tab_scan(wp mod nt)
   wbs = wb + nb * ws
   ind_bs = sort(wbs)
   wp = wp(ind_bs)
   wb = wb(ind_bs)
   ws = ws(ind_bs)
   wbs = wbs(ind_bs)
   wt = wp mod nt
   ind_bs = uniq(wbs)
   cbs = n_elements(ind_bs)

   weight_sample = weight(tab_scan(wp mod nt) + nscans * (wp / nt))

   tab_ind = [-1, ind_bs]
   tab_b = wb(ind_bs)
   tab_s = ws(ind_bs)
   for ibs = 0L, cbs - 1L do begin
      wplane = tab_ind(ibs) + 1 + lindgen(tab_ind(ibs+1) - tab_ind(ibs))
      vectpos_plane = wproj(k) + dimxy * tab_b(ibs) + dimxynb * tab_s(ibs)
      cube_weight(vectpos_plane) += total(weight_sample(wplane), /nan)
      cube_signal(vectpos_plane) += total(signal(wp(wplane)) * weight_sample(wplane), /nan)
      cube_subscan(vectpos_plane) = median(tab_subscan(wt(wplane)))
   endfor
endfor
wproj = 0
revind = 0

wcov = where(cube_weight gt 0.)
cube_signal(wcov) = cube_signal(wcov) / cube_weight(wcov)
wcov = 0
w = where(finite(weight) gt 0)
cube_weight = cube_weight / mean(weight(w))
w = where(cube_weight le 0.)
cube_signal(w) = !values.f_nan
w = 0



;;; determine whether the field is covered by largely disjoint scans or not:

scanrot_pos = list_scanrot
w = where(scanrot_pos lt 0.D0, cw)
if cw ge 1 then scanrot_pos(w) += 180.D0
if min(scanrot_pos) ge 90.D0 then scanrot_pos -= 90.D0

parity = 1 + intarr(nscans)
d = abs(scanrot_pos - scanrot_pos(0))
w = where(d lt 45.D0 or d gt 135.D0)
parity(w) = 0
if max(parity) eq 0 then begin
   w = where(indgen(nscans) mod 2 eq 1)
   parity(w) = 1
endif

weight_scans = total(cube_weight, 3, /nan)
w = where(weight_scans gt 0.)
weight_scans /= median(weight_scans(w))
weight_cross_even = 1
weight_cross_odd = 1
tab_cw = lonarr(nscans)
for i = 0, nscans - 1 do begin
   if parity(i) eq 0 then weight_cross_even *= weight_scans(*, *, i) else weight_cross_odd *= weight_scans(*, *, i)
   w = where(weight_scans(*, *, i) gt 0., cw)
   tab_cw(i) = cw
endfor
ind_even = where(parity eq 0, n_even)
ind_odd = where(parity eq 1, n_odd)
w_even = where(weight_cross_even gt 0., cwe)
w_odd = where(weight_cross_odd gt 0., cwo)
if (cwe lt min(tab_cw(ind_even)) / 4. or cwo lt min(tab_cw(ind_odd)) / 4.) then disjoint = 1

if (cwe lt 0.75 * max(tab_cw(ind_even)) or cwo lt 0.75 * max(tab_cw(ind_odd))) then begin
   inner_edges = 1
   print, 'taking care of increased noise on scan edges within central part of the map'
   print, 'area common to all scans / maximum scan area:'
   print, 'even scans: ' + chain(cwe / float(max(tab_cw(ind_even))), 2)
   print, 'odd scans: ' + chain(cwo / float(max(tab_cw(ind_odd))), 2)
endif
frac_overlap = min([cwe / float(max(tab_cw(ind_even))), cwo / float(max(tab_cw(ind_odd)))])

weight_cross_even = 0
weight_cross_odd = 0
w = 0


if keyword_set(disjoint) then begin

   ;;; determine scan order to have the maximum overlap between successive scans:
   tab_cw = lonarr(nscans, nscans)
   for j = 1, nscans - 1 do for i = 0, j - 1 do begin
      w_over = where(weight_scans(*, *, i) * weight_scans(*, *, j) gt 0., cw)
      tab_cw(i, j) = cw
   endfor
   w_over = 0
   tab_cw_init = tab_cw
   sum_overlap = intarr(nscans)
   for i = 0, nscans - 1 do sum_overlap(i) = total([tab_cw(0:i, i), reform(tab_cw(i, i:nscans-1))] gt 25L)
   m = max(sum_overlap, ind)
   ind_overlap_scans = [ind]
   ncur = 1
   while ncur lt nscans do begin
      tab_cw_check = lonarr(nscans, nscans)
      for k = 0, ncur - 1 do begin
         tab_cw_check(ind_overlap_scans(k), *) = tab_cw(ind_overlap_scans(k), *)
         tab_cw_check(*, ind_overlap_scans(k)) = tab_cw(*, ind_overlap_scans(k))
      endfor
      m = max(tab_cw_check, ind)
      if min(abs(ind_overlap_scans - (ind mod nscans))) eq 0 then ind_cur = ind / nscans else ind_cur = (ind mod nscans)
      ind_overlap_scans = [ind_overlap_scans, ind_cur]
      for k = 0, ncur - 1 do begin
         tab_cw(ind_overlap_scans(k), ind_cur) = 0L
         tab_cw(ind_cur, ind_overlap_scans(k)) = 0L
      endfor
      ncur += 1
   endwhile


   ;;; determine scan order to have both alternating angles and substantial overlap between even and odd scans:
   parity = !values.f_nan + intarr(nscans)
   tab_cw = tab_cw_init
   vect_weight = lonarr(nscans)
   for i = 0, nscans - 1 do begin
      w = where(weight_scans(*, *, i) gt 0, cw)
      vect_weight(i) = cw
   endfor
   m = max(tab_cw, ind)
   ind_cur = min([ind mod nscans, ind / nscans])
   parity(ind_cur) = 0
   ind_parity = [ind_cur]
   ncur = 1
   while ncur lt nscans do begin
      tab_cw_check = lonarr(nscans, nscans)
      for k = 0, ncur - 1 do begin
         tab_cw_check(ind_parity(k), *) = tab_cw(ind_parity(k), *)
         tab_cw_check(*, ind_parity(k)) = tab_cw(*, ind_parity(k))
      endfor
      m = max(tab_cw_check, ind)
      ind_cur = ind mod nscans
      if finite(parity(ind_cur)) lt 1 then ind_cur = ind / nscans
      tab_cw_check = lonarr(nscans, nscans)
      tab_cw_check(ind_cur, *) = tab_cw(ind_cur, *)
      tab_cw_check(*, ind_cur) = tab_cw(*, ind_cur)
      w = where(tab_cw_check gt vect_weight(ind_cur) / 4L, cw)
      if cw lt 1 then m = max(tab_cw_check, w)
      ind = [w mod nscans, w / nscans]
      ind = ind(uniq(ind, sort(ind)))
      w = where(ind eq ind_cur)
      ind = shift(ind, -w(0))
      if cw gt 1 then begin
         list_scanrot_cur = list_scanrot(ind)
         ind_sort = order_angles(list_scanrot_cur)
         ind = ind(ind_sort(0:1))
      endif
      ind_over = ind(1)
      d = abs(scanrot_pos(ind_over) - scanrot_pos(ind_parity(ncur-1)))
      if (d lt 45.D0 or d gt 135.D0) then parity(ind_over) = parity(ind_parity(ncur-1)) $
         else parity(ind_over) = (parity(ind_parity(ncur-1)) + 1) mod 2
      ind_parity = [ind_parity, ind_over]
      for k = 0, ncur - 1 do begin
         tab_cw(ind_parity(k), ind_over) = 0L
         tab_cw(ind_over, ind_parity(k)) = 0L
      endfor
      ncur += 1
   endwhile

endif else ind_overlap_scans = indgen(nscans)


;;; define the zone covered by both scans and cross-scans:
w_even = where(parity eq 0, c_even)
w_odd = where(parity eq 1, c_odd)
if c_even gt 1 then weight_cross = total(weight_scans(*, *, w_even), 3) $
   else weight_cross = weight_scans(*, *, w_even)
if c_odd gt 1 then weight_cross *= total(weight_scans(*, *, w_odd), 3) $
   else weight_cross *= weight_scans(*, *, w_odd)
weight_scans = 0



;;; exclude sources using the same mask as baselines.pro:

match_sparse = 0
if (keyword_set(disjoint) and frac_overlap le 0.01 and (not keyword_set(galactic)) $
 and size_deg ge size_lim_smallfield and (not keyword_set(smallfield))) then begin
   match_sparse = 1
   restore, file_maps
   w = where(map_weight gt 0., cw, complement=wz)
   ind_sort = reverse(sort(map_signal(w)))
   map_back(wz) = !values.f_nan
   map_back(w(ind_sort(0:cw/20))) = !values.f_nan
   ind_sort = 0
endif
if keyword_set(smallfield) then begin
   restore, file_maps
   w = where(map_weight gt 0., cw, complement=wz)
   ind_sort = reverse(sort(map_signal(w)))
   map_back = map_signal
   map_back(wz) = !values.f_nan
   map_back(w(ind_sort(0:cw/20))) = !values.f_nan
   ind_sort = 0
endif
map_mask = fltarr(dimx_iter, dimy_iter)
nx = (dimx / fact) * fact
ny = (dimy / fact) * fact
map_mask(0:dimx/fact-1, 0:dimy/fact-1) = rebin(map_back(0:nx-1, 0:ny-1), dimx/fact, dimy/fact)
w = where(finite(map_mask) lt 1 and weight_cross gt 0., cw)
if cw ge 1 then begin
   wz_init = reform(rebin(w, cw, nb) + (tab_nb ## (1L + lonarr(cw))), cw*nb)
   for s = 0L, nscans - 1 do begin
      wz = wz_init + dimxynb * s
      cube_weight(wz) = 0.
      cube_subscan(wz) = 0.
   endfor
endif

if not keyword_set(match_sparse) then map_back = 0
map_mask = 0
wz_init = 0
wz = 0



;;; create maps for single scans,
;;; and build reference maps for even and odd scans, excluding potentially deviant scans:

if (keyword_set(cut_start) and (nscans ge 4 or (not keyword_set(galactic)))) then begin
   wcut = where(cut_start gt 0, cw_cut)
   for c = 0, cw_cut - 1 do begin
      w = where(ind_subscans ge ind_scans(wcut(c)))
      subs = w(0)
      w = where(cube_subscan eq subs + 1, cw)
      if cw ge 1 then cube_weight(w) = 0.
   endfor
endif

map_scans = total(cube_signal * cube_weight, 3, /nan)
weight_scans = total(cube_weight, 3)
wdef_scans = where(weight_scans gt 0., complement=wz_scans)
map_scans(wdef_scans) /= weight_scans(wdef_scans)
map_scans(wz_scans) = !values.f_nan

flag_valid_scans = 1 + intarr(nscans)
if (not keyword_set(disjoint)) then if nscans gt 4 then begin
   nexc = 0
   ind_valid = where(flag_valid_scans gt 0 and parity eq 0, c_valid)
   map_median = median(map_scans(*, *, ind_valid), dimension=3, /even)
   dev = sqrt(median((map_scans(*, *, ind_valid) - rebin(map_median, dimx_iter, dimy_iter, c_valid))^2.))
   if c_valid gt 10 then dev *= sqrt(float(c_valid) / 10.)
   for s = 0, nscans - 1 do if (flag_valid_scans(s) gt 0 and parity(s) eq 0) then begin
      dev_scan = median(abs(map_scans(*, *, s) - map_median))
      if dev_scan gt 4. * dev then begin
         print, '     excluding deviant scan ' + chain(s) + ' from reference maps: ndev = ' + chain(dev_scan / dev, 1)
         flag_valid_scans(s) = 0
         nexc = 1
      endif
   endif
   ind_valid = where(flag_valid_scans gt 0 and parity eq 1, c_valid)
   map_median = median(map_scans(*, *, ind_valid), dimension=3, /even)
   dev = sqrt(median((map_scans(*, *, ind_valid) - rebin(map_median, dimx_iter, dimy_iter, c_valid))^2.))
   if c_valid gt 10 then dev *= sqrt(float(c_valid) / 10.)
   for s = 0, nscans - 1 do if (flag_valid_scans(s) gt 0 and parity(s) eq 1) then begin
      dev_scan = median(abs(map_scans(*, *, s) - map_median))
      if dev_scan gt 4. * dev then begin
         print, '     excluding deviant scan ' + chain(s) + ' from reference maps: ndev = ' + chain(dev_scan / dev, 1)
         flag_valid_scans(s) = 0
         nexc = 1
      endif
   endif
endif
ind_even = where(parity eq 0 and flag_valid_scans gt 0, n_even)
ind_odd = where(parity eq 1 and flag_valid_scans gt 0., n_odd)
ind_valid = where(flag_valid_scans gt 0)


if n_even ge 2 then begin
   ima_even = total(map_scans(*, *, ind_even) * weight_scans(*, *, ind_even), 3, /nan)
   weight_even = total(weight_scans(*, *, ind_even), 3)
   w_even = where(weight_even gt 0.)
   ima_even(w_even) /= weight_even(w_even)
endif else begin
   ima_even = map_scans(*, *, ind_even)
   weight_even = weight_scans(*, *, ind_even)
   w_even = where(weight_even gt 0.)
endelse
if n_odd ge 2 then begin
   ima_odd = total(map_scans(*, *, ind_odd) * weight_scans(*, *, ind_odd), 3, /nan)
   weight_odd = total(weight_scans(*, *, ind_odd), 3)
   w_odd = where(weight_odd gt 0.)
   ima_odd(w_odd) /= weight_odd(w_odd)
endif else begin
   ima_odd = map_scans(*, *, ind_odd)
   weight_odd = weight_scans(*, *, ind_odd)
   w_odd = where(weight_odd gt 0.)
endelse

weight_ref = total(weight_scans(*, *, ind_valid), 3)
wdef_ref = where(weight_ref gt 0., complement=wz_ref)



restore, file_pixcoords
indxy_coarse = floor(x_bolos / fact) + dimx_iter * floor(y_bolos / fact)
x_bolos = 0
y_bolos = 0

if inst eq 'spire' then begin
   min_nloop = 4
   max_nloop = 8
   nloop_switch = 2
   noise_level = median(maxnoise)
endif else begin
   min_nloop = 3
   max_nloop = 3
   if (keyword_set(smallfield) and size_deg lt 1.) then begin
      min_nloop = 4
      max_nloop = 4
   endif
   nloop_switch = 2
   noise_level = 3. * median(maxnoise)
endelse

ima_diff = fltarr(dimx_iter, dimy_iter)

simul = fltarr(nt, nb)



;;; first rectify each scan:

nloop = 4
if (keyword_set(galactic) or keyword_set(match_sparse)) then for loop = 0, nloop - 1 do begin
   for s = 0, nscans - 1 do begin

      if loop lt nloop / 2 then begin
         if parity(s) eq 0 then map_ref = ima_odd else map_ref = ima_even
      endif

      indxy_scan = indxy_coarse(ind_scans(s):ind_scans(s+1)-1, *)
      diff_scan = map_scans(dimxy*s+indxy_scan) - map_ref(indxy_scan)
      wb = where(finite(maxnoise(s, *)) gt 0)
      diff = total(diff_scan(*, wb), 2, /nan)
      norm = total(finite(diff_scan(*, wb)), 2)
      w = where(norm gt 0.)
      diff(w) /= norm(w)
      time_ref = time_s(ind_scans(s)+w(0))
      fit_scan = absdev_fit(float(-time_ref + time_s(ind_scans(s)+w)), diff(w), coef=coef, $
         noise_level=noise_level, /sym, /dyn, ns=ns)
      simul_scan = coef(0) + coef(1) * float(-time_ref + time_s(ind_scans(s):ind_scans(s+1)-1))
      simul(ind_scans(s):ind_scans(s+1)-1, *) += rebin(simul_scan, ind_scans(s+1)-ind_scans(s), nb)

      for b = 0L, nb - 1 do if finite(maxnoise(s, b)) gt 0 then begin
         ind_sort = sort(indxy_scan(*, b))
         ind_uniq = uniq(indxy_scan(ind_sort, b))
         nu = n_elements(ind_uniq)
         ind_mid = ([0, ind_uniq(0:nu-2)+1] + ind_uniq) / 2
         cube_signal(dimxy*b+dimxynb*s+indxy_scan(ind_sort(ind_mid), b)) -= simul_scan(ind_sort(ind_mid))
      endif
      ind_sort = 0
      ind_uniq = 0
      ind_mid = 0
   endfor

   if loop lt nloop - 1 then begin
      map_scans = total(cube_signal * cube_weight, 3, /nan)
      map_scans(wdef_scans) /= weight_scans(wdef_scans)
      map_scans(wz_scans) = !values.f_nan

      if loop lt nloop / 2 - 1 then begin
         if n_even ge 2 then begin
            ima_even = total(map_scans(*, *, ind_even) * weight_scans(*, *, ind_even), 3, /nan)
            ima_even(w_even) /= weight_even(w_even)
         endif else ima_even = map_scans(*, *, ind_even)
         if n_odd ge 2 then begin
            ima_odd = total(map_scans(*, *, ind_odd) * weight_scans(*, *, ind_odd), 3, /nan)
            ima_odd(w_odd) /= weight_odd(w_odd)
         endif else ima_odd = map_scans(*, *, ind_odd)
      endif else begin
         map_ref = total(map_scans(*, *, ind_valid) * weight_scans(*, *, ind_valid), 3, /nan)
         map_ref(wdef_ref) /= weight_ref(wdef_ref)
         map_ref(wz_ref) = !values.f_nan
      endelse
   endif
endfor

ima_even = 0
ima_odd = 0



;;; then destripe by segments of 4 subscans:

nloop = 2
nsubs_crit = 4
if (keyword_set(galactic) or keyword_set(match_sparse)) then for loop = 0, nloop - 1 do begin

   if size_deg gt size_lim_ref then begin
      map_scans = total(cube_signal * cube_weight, 3, /nan)
      map_scans(wdef_scans) /= weight_scans(wdef_scans)
      map_scans(wz_scans) = !values.f_nan
   endif else begin
      ima_signal = total(total(cube_signal(*, *, *, ind_valid) $
         * cube_weight(*, *, *, ind_valid), 4, /nan), 3, /nan)
      ima_signal(wdef_ref) /= weight_ref(wdef_ref)
      ima_signal(wz_ref) = !values.f_nan
   endelse

   for s = 0, nscans - 1 do begin
      wsubs = where(ind_subscans ge ind_scans(s) and ind_subscans le ind_scans(s+1) - 1, csubs)
      w = where(finite(maxnoise(s, *)) gt 0, nb_def)

      nseg = round(csubs / float(nsubs_crit))
      if nseg gt 1 then begin
         nsubs_cut = max([2 * round(csubs / float(nseg) / 2.), 2])
         nseg = ceil(csubs / float(nsubs_cut))
         ind_start = ind_subscans(wsubs(nsubs_cut*lindgen(nseg)))
         ind_end = [ind_start(1:nseg-1) - 1, ind_scans(s+1) - 1]
         subs_1 = wsubs(nsubs_cut*lindgen(nseg))
         subs_2 = [subs_1(1:nseg-1) - 1, wsubs(csubs-1)]
      endif else begin
         nseg = 1
         ind_start = [ind_scans(s)]
         ind_end = [ind_scans(s+1) - 1]
         subs_1 = [wsubs(0)]
         subs_2 = [wsubs(csubs-1)]
         nsubs_cut = csubs
      endelse

      if size_deg gt size_lim_ref then begin
         if parity(s) eq 0 then begin
            if n_odd ge 2 then begin
               ima_signal = total(map_scans(*, *, ind_odd) * weight_scans(*, *, ind_odd), 3, /nan)
               ima_signal(w_odd) /= weight_odd(w_odd)
            endif else ima_signal = map_scans(*, *, ind_odd)
         endif else begin
            if n_even ge 2 then begin
               ima_signal = total(map_scans(*, *, ind_even) * weight_scans(*, *, ind_even), 3, /nan)
               ima_signal(w_even) /= weight_even(w_even)
            endif else ima_signal = map_scans(*, *, ind_even)
         endelse
      endif

      for seg = 0, nseg - 1 do begin
         indxy_scan = indxy_coarse(ind_start(seg):ind_end(seg), *)
         time_scan = float(-time_s(ind_start(seg)) + time_s(ind_start(seg):ind_end(seg)))
         wdef_fit = where(vscan(ind_start(seg):ind_end(seg)) gt vscan_as_s / 20.)
         w = where(total(weight_cross(indxy_scan) gt 0., 2) gt nb_def / 2L, cdef_fit)
         w = 0
         indxy_fit = indxy_scan(wdef_fit, *)
         time_fit = time_scan(wdef_fit)

         for b = 0L, nb - 1L do if finite(maxnoise(s, b)) gt 0 then begin
            wxy = where(cube_subscan(*, *, b, s) ge subs_1(seg) + 1 $
               and cube_subscan(*, *, b, s) le subs_2(seg) + 1, cxy)
            if cxy ge 100 then begin
               ima_diff += !values.f_nan
               ima_diff(wxy) = cube_signal(dimxy*b+dimxynb*s+wxy) - ima_signal(wxy)
               w = where(finite(ima_diff(indxy_fit(*, b))) gt 0, cdef)
               w = 0
               if cdef gt max([cdef_fit / 2L, nsubs_cut * 100L]) then begin
                  fit_scan = absdev_fit(time_fit, ima_diff(indxy_fit(*, b)), coef=coef, noise_level=noise_level, $
                     /sym, /dyn, ns=ns)
                  simul_scan = coef(0) + coef(1) * time_scan
                  simul(ind_start(seg):ind_end(seg), b) += simul_scan

                  if (keyword_set(debug) and b eq bref and seg eq nseg / 2) then begin
                     time_2 = systime(/seconds)
                     t_proc += (time_2 - time_1)
                     wset, 0
                     w = where(finite(ima_diff(indxy_fit(*, b))) gt 0)
                     otherplot, time_fit(w), ima_diff(indxy_fit(w, b)), xtitle='time (s)', $
                        ytitle='signal (Jy/bolometer)', title='bolometer '+chain(bref) $
                        +', scan '+chain(s)+', segment '+chain(seg)+': observed - simulated from map', psym=-7
                     otherplot, /over, color=10, time_scan, simul_scan
                     res = dialog_message('   Continue ?   ', /center, /information, title='Pause')
                     time_1 = systime(/seconds)
                  endif

                  ind_sort = sort(indxy_scan(*, b))
                  ind_uniq = uniq(indxy_scan(ind_sort, b))
                  nu = n_elements(ind_uniq)
                  ind_mid = ([0, ind_uniq(0:nu-2)+1] + ind_uniq) / 2
                  cube_signal(dimxy*b+dimxynb*s+indxy_scan(ind_sort(ind_mid), b)) -= simul_scan(ind_sort(ind_mid))
               endif
            endif
         endif
         ind_sort = 0
         ind_uniq = 0
         ind_mid = 0

      endfor
   endfor
endfor
indxy_scan = 0



;;; then destripe subscan by subscan:

amp_simul = fltarr(nsubscans)
amp_error = fltarr(nsubscans)
error_fit = fltarr(nsubscans, nb)
ind_excl = intarr(nsubscans)
flag_bol = intarr(nsubscans, nb)

nt_seg = 500L
nt_off = nt_seg / 10L


nloop = 0
repeat begin
   if (size_deg gt size_lim_ref and nloop lt nloop_switch) then begin
      map_scans = total(cube_signal * cube_weight, 3, /nan)
      map_scans(wdef_scans) /= weight_scans(wdef_scans)
      map_scans(wz_scans) = !values.f_nan
   endif else begin
      ima_signal = total(total(cube_signal(*, *, *, ind_valid) $
         * cube_weight(*, *, *, ind_valid), 4, /nan), 3, /nan)
      ima_signal(wdef_ref) /= weight_ref(wdef_ref)
      ima_signal(wz_ref) = !values.f_nan
   endelse

   flag_bol_cur = 0
   for s = 0, nscans - 1 do begin
      wsubs = where(ind_subscans ge ind_scans(s) and ind_subscans le ind_scans(s+1) - 1, csubs)
      subs_visu = wsubs(csubs / 2)
      w = where(finite(maxnoise(s, *)) gt 0, nb_def)

      if (size_deg gt size_lim_ref and nloop lt nloop_switch) then begin
         if parity(s) eq 0 then begin
            if n_odd ge 2 then begin
               ima_signal = total(map_scans(*, *, ind_odd) * weight_scans(*, *, ind_odd), 3, /nan)
               ima_signal(w_odd) /= weight_odd(w_odd)
            endif else ima_signal = map_scans(*, *, ind_odd)
         endif else begin
            if n_even ge 2 then begin
               ima_signal = total(map_scans(*, *, ind_even) * weight_scans(*, *, ind_even), 3, /nan)
               ima_signal(w_even) /= weight_even(w_even)
            endif else ima_signal = map_scans(*, *, ind_even)
         endelse
      endif

      for subs = wsubs(0), wsubs(csubs-1) do begin
         nsubs = ind_subscans(subs+1) - ind_subscans(subs)
         indxy_subs = indxy_coarse(ind_subscans(subs):ind_subscans(subs+1)-1, *)
         simul_subs = fltarr(ind_subscans(subs+1)-ind_subscans(subs), nb)
         time_subs = float(-time_s(ind_subscans(subs)) + time_s(ind_subscans(subs):ind_subscans(subs+1)-1))
         wdef_fit = where(vscan(ind_subscans(subs):ind_subscans(subs+1)-1) gt vscan_as_s / 20., cdef_ini)
         w = where(total(weight_cross(indxy_subs) gt 0., 2) gt nb_def / 2L, cdef_fit)
         w = 0
         indxy_fit = indxy_subs(wdef_fit, *)
         time_fit = time_subs(wdef_fit)
         dt = max([10. * timestep, time_subs(nsubs-1L) / 20.])


         ;;; break down deviant or very long subscans into several pieces:

         if ((ind_excl(subs) gt 0 or (nloop eq max_nloop - 1 and inst eq 'pacs')) $
            and cdef_fit gt 2L * nt_seg and (not keyword_set(galactic))) then begin
            slice_diff_aver = !values.f_nan + fltarr(nsubs, nb)
            for b = 0L, nb - 1L do if finite(maxnoise(s, b)) gt 0 then begin
               wxy = where(cube_subscan(*, *, b, s) eq subs + 1, cxy)
               if cxy ge 10 then begin
                  ima_diff += !values.f_nan
                  ima_diff(wxy) = cube_signal(dimxy*b+dimxynb*s+wxy) - ima_signal(wxy)
                  slice_diff_aver(wdef_fit, b) = ima_diff(indxy_fit(*, b))
               endif
            endif
            weight_diff_aver = total(finite(slice_diff_aver), 2)
            slice_diff_aver = total(slice_diff_aver, 2, /nan) / weight_diff_aver

            nseg = floor(nsubs / nt_seg)
            fit_subscan = !values.f_nan + fltarr(nsubs)
            for kseg = 0L, nseg - 1L do begin
               it_i = kseg * nt_seg
               it_f = (kseg + 1) * nt_seg - 1
               if kseg eq nseg - 1 then it_f = nsubs - 1
               wdest = where(total(weight_cross(indxy_subs(it_i:it_f, *)) gt 0., 2) gt nb_def / 2L $
                  and finite(slice_diff_aver(it_i:it_f)) gt 0, cdef_seg)
               if cdef_seg gt nt_seg / 2L then begin
                  if kseg gt 0L then off_i = nt_off else off_i = 0
                  if kseg lt nseg - 1L then off_f = nt_off else off_f = 0
                  time_seg = time_subs(it_i:it_f)
                  fit_seg = absdev_fit(time_seg(wdest), slice_diff_aver(it_i+wdest), coef=coef, noise_level=noise_level, /sym, /dyn, ns=ns)
                  fit_subscan(it_i+wdest(0)+off_i:it_i+wdest(cdef_seg-1L)-off_f) = coef(0) + $
                     coef(1) * time_seg(wdest(0)+off_i:wdest(cdef_seg-1L)-off_f)
               endif
            endfor
            wz = where(finite(fit_subscan) lt 1, complement=wdef, ncomplement=cdef)
            if cdef gt cdef_fit / 2L then begin
               fit_subscan(wz) = interpol([0., 0., fit_subscan(wdef), 0., 0.], $
                  [time_subs(0) - dt - timestep, time_subs(wdef(0)) - dt, time_subs(wdef), $
                  time_subs(wdef(cdef-1)) + dt, time_subs(nsubs-1) + dt + timestep], $
                  time_subs(wz))
               for b = 0L, nb - 1L do if finite(maxnoise(s, b)) gt 0 then simul_subs(*, b) += fit_subscan
            endif
            wz = 0
            wdef = 0
         endif


         if (nloop lt max_nloop - 1 or cdef_fit le 2L * nt_seg or keyword_set(galactic) or ind_excl(subs) gt 0 or inst ne 'pacs') then $
          for b = 0L, nb - 1L do if finite(maxnoise(s, b)) gt 0 then begin
            wxy = where(cube_subscan(*, *, b, s) eq subs + 1, cxy)
            if cxy ge 10 then begin
               ima_diff += !values.f_nan
               ima_diff(wxy) = cube_signal(dimxy*b+dimxynb*s+wxy) - ima_signal(wxy)
               wdef = where(finite(ima_diff(indxy_fit(*, b))) gt 0 and weight_cross(indxy_fit(*, b)) gt 0., cdef)
               if cdef gt cdef_ini / 2L then begin
                  diff_fit = ima_diff(indxy_fit(*, b)) - simul_subs(wdef_fit, b)
                  wz = where(weight_cross(indxy_fit(*, b)) le 0., cz, complement=wc)
                  if cz ge 1 then diff_fit(wz) = !values.f_nan
                  fit_subs = absdev_fit(time_fit, diff_fit, coef=coef, noise_level=noise_level, /sym, /dyn, ns=ns)
                  werr = where(finite(diff_fit) gt 0)
                  error_fit(subs, b) = sqrt(mean((diff_fit(werr) - fit_subs(werr))^2.))
                  werr = where(finite(diff_fit) gt 0 and abs(diff_fit - fit_subs) lt 6. * error_fit(subs, b))
                  error_fit(subs, b) = sqrt(mean((diff_fit(werr) - fit_subs(werr))^2.))
                  fit_subscan = !values.f_nan + fltarr(nsubs)
                  fit_subscan(wdef_fit(wdef(0):wdef(cdef-1L))) = coef(0) + coef(1) * time_fit(wdef(0):wdef(cdef-1L))
                  wz = where(finite(fit_subscan) lt 1, cz, complement=wdef, ncomplement=cdef)
                  if cz ge 1 then fit_subscan(wz) = interpol([0., 0., fit_subscan(wdef), 0., 0.], $
                     [time_subs(0) - dt - timestep, time_subs(wdef(0)) - dt, time_subs(wdef), $
                     time_subs(wdef(cdef-1)) + dt, time_subs(nsubs-1) + dt + timestep], $
                     time_subs(wz))
                  wz = 0
                  wdef = 0
                  simul_subs(*, b) += fit_subscan

                  if (keyword_set(debug) and subs eq subs_visu and b eq bref) then begin
                     time_2 = systime(/seconds)
                     t_proc += (time_2 - time_1)
                     wset, 0
                     w = where(finite(ima_diff(indxy_fit(*, b))) gt 0)
                     otherplot, time_fit(w), ima_diff(indxy_fit(w, b)), xtitle='time (s)', $
                        ytitle='signal (Jy/bolometer)', title='bolometer '+chain(bref) $
                        +', subscan '+chain(subs)+': observed - simulated from map', psym=-7
                     otherplot, /over, color=10, time_subs, simul_subs(*, b)
                     res = dialog_message('   Continue ?   ', /center, /information, title='Pause')
                     time_1 = systime(/seconds)
                  endif

                  ind_sort = sort(indxy_subs(*, b))
                  ind_uniq = uniq(indxy_subs(ind_sort, b))
                  nu = n_elements(ind_uniq)
                  ind_mid = ([0, ind_uniq(0:nu-2)+1] + ind_uniq) / 2
                  cube_signal(dimxy*b+dimxynb*s+indxy_subs(ind_sort(ind_mid), b)) -= simul_subs(ind_sort(ind_mid), b)
               endif else begin
                  error_fit(subs, b) = 0.
                  simul_subs(*, b) = 0.
               endelse
            endif else begin
               error_fit(subs, b) = 0.
               simul_subs(*, b) = 0.
            endelse
         endif else begin
            error_fit(subs, b) = 0.
         endelse
         ind_sort = 0
         ind_uniq = 0
         ind_mid = 0

         simul(ind_subscans(subs):ind_subscans(subs+1)-1, *) += simul_subs
         wb = where(error_fit(subs, *) gt 0., cb)
         if cb ge 1 then begin
            amp_simul(subs) = median(sqrt(total((simul_subs(*, wb) - (median(simul_subs(*, wb), dimension=1) ## (fltarr(nsubs) + 1.)))^2., 1) $
               / float(nsubs)) / maxnoise(s, wb), /even)
            amp_error(subs) = median(error_fit(subs, wb) / maxnoise(s, wb), /even)
         endif else amp_error(subs) = 0.
         if (cb ge 0.7 * nb and nloop le 1) then begin
            m = median(error_fit(subs, wb), /even)
            dev = sqrt(mean((error_fit(subs, wb) - m)^2.))
            for k = 1, 9 do begin
               w = where(error_fit(subs, wb) lt m + 3. * dev)
               dev = sqrt(mean((error_fit(subs, wb(w)) - m)^2.))
            endfor
            w = where(error_fit(subs, *) gt m + 100. * dev and flag_bol(subs, *) lt 1, cw)
            if cw ge 1 then begin
               flag_bol_cur = 1
               flag_bol(subs, w) = 1
               signal(ind_subscans(subs):ind_subscans(subs+1)-1, w) = !values.f_nan
               for ib = 0L, cw - 1L do cube_weight(indxy_subs(*,w(ib))+dimxy*w(ib)+dimxynb*s) = 0.
               if keyword_set(visu) then print, '     subscan ' + chain(subs) + ': excluding bolometer ' $
                  + chain(w) + ' (ndev = ' + chain((error_fit(subs, w) - m) / dev, 1) + ')'
            endif
         endif
      endfor
   endfor


   if (not keyword_set(disjoint)) then if nloop lt max_nloop - 1 then begin

      if (nloop le 1 and flag_bol_cur eq 1) then begin
         w = where(total(flag_bol, 1) ge nsubscans / 2. and total(flag_bol, 1) lt nsubscans, cw)
         if cw ge 1 then begin
            flag_bol(*, w) = 1
            signal(*, w) = !values.f_nan
            for ib = 0L, cw - 1L do cube_weight(*, *, w(ib), *) = 0.
            if keyword_set(visu) then print, '     excluding bolometer ' + chain(w) + ' from all subscans'
         endif
      endif

      if (nscans lt 4 and size_deg lt 1.) then thres_dev = 5. else thres_dev = 3.5
      if (nloop ge nloop_switch or keyword_set(smallfield)) then thres_dev = 6.
      amp_dev = amp_error
      if (dimx gt 1.5 * dimy or dimy gt 1.5 * dimx) then flag_rect = 1 else flag_rect = 0
      if flag_rect eq 1 then ndev = fltarr(nsubscans)
      for s = 0, nscans - 1 do begin
         wsubs = where(ind_subscans ge ind_scans(s) and ind_subscans lt ind_scans(s+1) and amp_error gt 0., csubs)
         if csubs lt 1 then begin
            print, ''
            print, 'not enough resolution elements !'
            print, 'Use the /minimap option if the observations'
            print, 'were taken in the small-map or mini-map mode,'
            print, 'or if the field size is <~ 0.1 degree.'
            print, 'Otherwise, please check the mask settings in HIPE.'
            print, 'Exit.'
            retall
         endif
         if flag_rect eq 0 then begin
            med_amp = median(amp_error(wsubs), /even)
            amp_dev(wsubs) -= med_amp
         endif
         if flag_rect eq 1 then begin
            wsubs_def = where(amp_error(wsubs) gt 0.)
            med_amp = median(amp_error(wsubs(wsubs_def)), /even)
            amp_dev(wsubs) -= med_amp
            sig_amp = sqrt(mean(amp_dev(wsubs(wsubs_def))^2.))
            wsubs_def = where(amp_error(wsubs) gt 0. and amp_dev(wsubs) lt 3. * sig_amp)
            sig_amp = sqrt(mean(amp_dev(wsubs(wsubs_def))^2.))
            ndev(wsubs) = amp_dev(wsubs) / sig_amp
         endif
      endfor
      wsubs = where(amp_error gt 0.)
      if flag_rect eq 0 then begin
         sig_amp = sqrt(mean(amp_dev(wsubs)^2.))
         wsubs_def = where(amp_dev(wsubs) lt 3. * sig_amp)
         sig_amp = sqrt(mean(amp_dev(wsubs(wsubs_def))^2.))
         ndev = amp_dev / sig_amp
      endif
      w = where((abs(ndev) gt thres_dev or amp_error le 0.) and ind_excl lt 1, cw)
      if cw ge 1 then begin
         print, '     excluding deviant subscan ' + chain(w) + ' from reference map: ndev = ' + chain(ndev(w), 1) + crs
         ind_excl(w) = 1
         if keyword_set(debug) then begin
            time_2 = systime(/seconds)
            t_proc += (time_2 - time_1)
            wset, 0
            otherplot, lindgen(nsubscans), amp_dev, psym=4, xtitle='subscan index', ytitle='normalized error'
            otherplot, /over, lstyle=2, color=10, lindgen(nsubscans), fltarr(nsubscans) + 2. * sig_amp
            otherplot, /over, lstyle=2, color=10, lindgen(nsubscans), fltarr(nsubscans) + 3. * sig_amp
            otherplot, /over, lstyle=2, color=10, lindgen(nsubscans), fltarr(nsubscans) + 4. * sig_amp
            if cw ge 1 then otherplot, /over, color=10, w, amp_dev(w), psym=4
            res = dialog_message('   Continue ?   ', /center, /information, title='Pause')
            time_1 = systime(/seconds)
         endif
      endif
      for ks = 0L, cw - 1L do begin
         subs = w(ks)
         wz = where(cube_subscan eq subs + 1, cz)
         if cz ge 1 then cube_weight(wz) = 0.
      endfor
      wz = 0
      if (cw ge 1 or keyword_set(flag_bol_cur)) then begin
         weight_scans = total(cube_weight, 3)
         wdef_scans = where(weight_scans gt 0., complement=wz_scans)
         weight_ref = total(weight_scans(*, *, ind_valid), 3)
         wdef_ref = where(weight_ref gt 0., complement=wz_ref)
      endif
   endif

   nloop += 1
   if keyword_set(debug) then print, mean(amp_error(wsubs)), mean(amp_simul(wsubs))
endrep until ((nloop ge min_nloop and (mean(amp_error(wsubs)) le 1. or mean(amp_simul(wsubs)) le 0.1)) or nloop ge max_nloop)



;;; then update signal and drift:

for s = 0, nscans - 1 do begin
   time_ref = time_s(ind_scans(s))
   simul_scan = simul(ind_scans(s):ind_scans(s+1)-1, *)
   wnza = where(flag_nza(ind_scans(s):ind_scans(s+1)-1) eq 0, cnza, complement=wdef, ncomplement=cdef)
   if cnza ge 1 then begin
      tab_simul = transpose([simul_scan(wdef(0), *), simul_scan(wdef, *), simul_scan(wdef(cdef-1), *)])
      tab_time = float(-time_ref + [time_s(ind_scans(s))-maxtimediff, time_s(ind_scans(s)+wdef), time_s(ind_scans(s+1)-1)+maxtimediff])
      ind_time = interpol(findgen(cdef+2), tab_time, float(-time_ref + time_s(ind_scans(s):ind_scans(s+1)-1)))
      simul_scan = transpose(interpolate(tab_simul, ind_time))
   endif
   signal(ind_scans(s):ind_scans(s+1)-1, *) -= simul_scan
   cscan = ind_scans(s+1) - ind_scans(s)
   tab_simul = transpose([simul_scan(0, *), simul_scan, simul_scan(cscan-1, *)])
   tab_time = float(-time_ref + [time_s(ind_scans(s))-maxtimediff, time_s(ind_scans(s):ind_scans(s+1)-1), time_s(ind_scans(s+1)-1)+maxtimediff])
   ind_time = interpol(findgen(cscan+2), tab_time, -float(time_ref) + time_save(ind_sc_coarse(s):ind_sc_coarse(s+1)-1))
   drifts_all(ind_sc_coarse(s):ind_sc_coarse(s+1)-1, *) += transpose(interpolate(tab_simul, ind_time))
endfor


;;; match brightness of distinct observations with small overlap
;;;    or observations of big fields made in several slices:

updated_proj = 0
if ((keyword_set(disjoint) and frac_overlap gt (Machar()).eps) or keyword_set(match_sparse)) then begin
   save, file=file_signal, signal, /xdr
   tab_match = fltarr(nscans)

   ;;; first create single scan maps:
   dimxy = dimx * dimy
   map_scans = fltarr(dimx, dimy, nscans)
   weight_scans = fltarr(dimx, dimy, nscans)
   error_scans = fltarr(dimx, dimy, nscans)
   for i = 0, nscans - 1 do begin
      prepare_proj_fine, file_scalars=file_scalars, scan_next=i+1, ind_scans=ind_scans
      proj_map_nearest, map_signal, map_weight, map_error, file_scalars=file_scalars, maxnoise=maxnoise, $
         ind_scans=ind_scans, /comp_errors
      map_scans(*, *, i) = map_signal
      weight_scans(*, *, i) = map_weight
      error_scans(*, *, i) = map_error
   endfor
   prepare_proj_fine, file_scalars=file_scalars, ind_scans=ind_scans
   updated_proj = 1

   ;;; then match brightness of successive scans:
   for k = 1, nscans - 1 do begin
      i = ind_overlap_scans(k)
      if k gt 1 then begin
         map_weight_combined = total(weight_scans(*, *, ind_overlap_scans(0:k-1)), 3)
         map_signal_combined = total(map_scans(*, *, ind_overlap_scans(0:k-1)) * weight_scans(*, *, ind_overlap_scans(0:k-1)), 3) $
            / map_weight_combined
         map_error_combined = total(error_scans(*, *, ind_overlap_scans(0:k-1)) * weight_scans(*, *, ind_overlap_scans(0:k-1)), 3) $
            / map_weight_combined
      endif else begin
         map_weight_combined = weight_scans(*, *, ind_overlap_scans(0))
         map_signal_combined = map_scans(*, *, ind_overlap_scans(0))
         map_error_combined = error_scans(*, *, ind_overlap_scans(0))
      endelse
      w_over = where(weight_scans(*, *, i) * map_weight_combined gt 0.)
      m = min([median(weight_scans(i*dimxy+w_over)), median(map_weight_combined(w_over))])
      w_over = where(weight_scans(*, *, i) gt m / 2. and map_weight_combined gt m / 2.)
      m = min([median(weight_scans(i*dimxy+w_over)), median(map_weight_combined(w_over))])
      w_over = where(weight_scans(*, *, i) gt 0.7 * m and map_weight_combined gt 0.7 * m $
         and error_scans(*, *, i) lt 2. * map_error_combined and map_error_combined lt 2. * error_scans(*, *, i) $
         and finite(map_back) gt 0., c_over)
      if c_over ge 1000L then tab_match(i) = median(map_scans(i*dimxy+w_over) - map_signal_combined(w_over))
      if keyword_set(match_sparse) then begin
         w = where(weight_scans(*, *, i) gt 0.7 * m and finite(map_back) gt 0., c_i)
         if c_over lt 0.2 * c_i then begin
            weightmap = weight_scans(*, *, i)
            w = where(weight_scans(*, *, i) gt 0.7 * m and finite(map_back) gt 0., c_i, complement=wz)
            weightmap(wz) = 0.
            stat_map, med, sig, map=map_scans(*, *, i), weightmap=weightmap, noise_level=median(maxnoise)
            weightmap = map_weight_combined
            w_comb = where(map_weight_combined gt 0.7 * m and finite(map_back) gt 0., complement=wz)
            weightmap(wz) = 0.
            stat_map, med_comb, sig_comb, map=map_signal_combined, weightmap=weightmap, noise_level=median(maxnoise)
            tab_match(i) = med - med_comb
         endif
      endif
      if keyword_set(visu) then print, 'offset of scan ' + chain(i) + ' wrt preceding scan(s) = ' + chain(tab_match(i), 5)
      map_scans(*, *, i) -= tab_match(i)
   endfor
   for i = 1, nscans - 1 do signal(ind_scans(i):ind_scans(i+1)-1, *) -= tab_match(i)

endif



if (not keyword_set(galactic)) then begin
   w = where(finite(signal) gt 0)
   med = median(signal(w))
   sig = sqrt(mean((signal(w) - med)^2.))
   for k = 0, 9 do begin
      wl = where(signal(w) - med lt 2.5 * sig)
      med = median(signal(w(wl)))
      sig = sqrt(mean((signal(w(wl)) - med)^2.))
   endfor
endif else begin
   nsamp = nb * (2L + ceil(nt / 1000))
   w = where(finite(signal) gt 0)
   ind_sort = sort(signal(w))
   med = median(signal(w(ind_sort(0:nsamp))))
endelse
signal -= med


save, file=file_signal, signal, /xdr

prepare_proj_coarse, file_scalars=file_scalars
if (max(flag_bol) gt 0 and (not keyword_set(updated_proj))) then $
   prepare_proj_fine, file_scalars=file_scalars, ind_scans=ind_scans

end
